<?php

namespace Drupal\commerce_cmi;

/**
 * Provides CMI-related services.
 */
class CmiService {

  /**
   * Generates or validates the hash for CMI transaction.
   *
   * @param array $data
   *   The data to be hashed.
   * @param string $store_key
   *   The store key for hashing.
   *
   * @return string
   *   The generated hash.
   */
  public function generateHash(array $data, string $store_key): string {
    $hashParams = array_keys($data);
    natcasesort($hashParams);
    $hashval = "";
    foreach ($hashParams as $param) {
      $paramValue = trim($data[$param]);
      $escapedParamValue = str_replace("|", "\\|", str_replace("\\", "\\\\", $paramValue));

      $lowerParam = strtolower($param);
      if ($lowerParam != "hash" && $lowerParam != "encoding") {
        $hashval = $hashval . $escapedParamValue . "|";
      }
    }

    $escapedStoreKey = str_replace("|", "\\|", str_replace("\\", "\\\\", $store_key));
    $hashval = $hashval . $escapedStoreKey;
    $calculatedHashValue = hash('sha512', $hashval);
    $hash = base64_encode(pack('H*', $calculatedHashValue));

    return $hash;
  }

  /**
   * Validates the hash received from CMI.
   *
   * @param array $post_data
   *   The data received from CMI in the callback.
   * @param string $store_key
   *   The store key used for hash calculation.
   *
   * @return bool
   *   Returns TRUE if the hash matches the received hash, FALSE otherwise.
   */
  public function validateHash(array $post_data, string $store_key): bool {
    $calculatedHash = $this->generateHash($post_data, $store_key);
    return $calculatedHash === $post_data['HASH'];
  }

}
