<?php

namespace Drupal\commerce_cmi\Plugin\Commerce\PaymentGateway;

use Drupal\Core\Form\FormStateInterface;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;

/**
 * Provides the CMI offsite Checkout payment gateway.
 *
 * @CommercePaymentGateway(
 *   id = "cmi",
 *   label = "CMI",
 *   display_label = "CMI",
 *   forms = {
 *     "offsite-payment" =
 *   "Drupal\commerce_cmi\PluginForm\OffsiteRedirect\CmiOffsiteForm",
 *   },
 * )
 */
class Cmi extends OffsitePaymentGatewayBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'clientid' => '',
      'storekey' => '',
      'api_url' => '',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['clientid'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Client ID'),
      '#description' => $this->t('Your CMI client ID.'),
      '#default_value' => $this->configuration['clientid'],
      '#required' => TRUE,
    ];

    $form['storekey'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Store Key'),
      '#description' => $this->t('Your CMI store key.'),
      '#default_value' => $this->configuration['storekey'],
      '#required' => TRUE,
    ];

    $form['api_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API URL'),
      '#description' => $this->t('The URL for the CMI API.'),
      '#default_value' => $this->configuration['api_url'],
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    if (!$form_state->getErrors()) {
      $values = $form_state->getValue($form['#parents']);
      $this->configuration['clientid'] = $values['clientid'];
      $this->configuration['storekey'] = $values['storekey'];
      $this->configuration['api_url'] = $values['api_url'];
    }
  }

}
