<?php

namespace Drupal\commerce_cmi\PluginForm\OffsiteRedirect;

use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\commerce_cmi\CmiService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides the Off-site payment form for CMI.
 */
class CmiOffsiteForm extends PaymentOffsiteForm {

  /**
   * The CMI service.
   *
   * @var \Drupal\commerce_cmi\CmiService
   */
  protected $cmiService;

  /**
   * Constructs a new CmiOffsiteForm.
   *
   * @param \Drupal\commerce_cmi\CmiService $cmi_service
   *   The CMI service.
   */
  public function __construct(CmiService $cmi_service) {
    $this->cmiService = $cmi_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('commerce_cmi.cmi_service')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $this->entity;
    /** @var \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface $payment_gateway_plugin */
    $payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();
    $configuration = $payment_gateway_plugin->getConfiguration();
    /** @var \Drupal\commerce_order\Entity\OrderInterface $order */
    $order = $payment->getOrder();
    /** @var \Drupal\address\Plugin\Field\FieldType\AddressItem $billing_address */
    $billing_address = $order->getBillingProfile()->address->first();

    $data = [
      'clientid' => $configuration['clientid'],
      'amount' => $payment->getAmount()->getNumber(),
      'oid' => $order->id() . '-' . $payment->id(),
      'okUrl' => $form['#return_url'],
      'failUrl' => $form['#cancel_url'],
      'TranType' => 'PreAuth',
      'callbackUrl' => Url::fromRoute('commerce_cmi.callback', [], ['absolute' => TRUE])->toString(),
      'shopurl' => Url::fromRoute('<front>', [], ['absolute' => TRUE])->toString(),
      'currency' => $payment->getAmount()->getCurrencyCode(),
      'rnd' => microtime(),
      'storetype' => '3D_PAY_HOSTING',
      'hashAlgorithm' => 'ver3',
      'lang' => \Drupal::languageManager()->getCurrentLanguage()->getId(),
      'BillToName' => $billing_address->getGivenName() . ' ' . $billing_address->getFamilyName(),
      'BillToCompany' => $billing_address->getOrganization(),
      'BillToStreet12' => $billing_address->getAddressLine1() . ' ' . $billing_address->getAddressLine2(),
      'BillToCity' => $billing_address->getLocality(),
      'BillToStateProv' => $billing_address->getAdministrativeArea(),
      'BillToPostalCode' => $billing_address->getPostalCode(),
      'BillToCountry' => '504',
      'email' => $order->getEmail(),
      'encoding' => 'UTF-8',
    ];

    // Add phone number if available.
    if ($order->hasField('field_phone') && !$order->get('field_phone')->isEmpty()) {
      $data['tel'] = $order->get('field_phone')->value;
    }
    else {
      $data['tel'] = '';
    }

    // Generate HASH.
    $data['HASH'] = $this->cmiService->generateHash($data, $configuration['storekey']);

    return $this->buildRedirectForm($form, $form_state, $configuration['api_url'], $data, 'post');
  }

}
